--[[
/*******************************************************************************\
*                                                                               *
* Levitator.lua                                                                 *
*                                                                               *
*********************************************************************************
* Licensed under the MIT or X11 License                                         *
*                                                                               *
* Copyright (c) 2016 Eisbearg                                                   *
*                                                                               *
* Permission is hereby granted, free of charge, to any person obtaining a copy  *
* of this software and associated documentation files (the "Software"), to deal *
* in the Software without restriction, including without limitation the rights  *
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell     *
* copies of the Software, and to permit persons to whom the Software is         *
* furnished to do so, subject to the following conditions:                      *
*                                                                               *
* The above copyright notice and this permission notice shall be included in    *
* all copies or substantial portions of the Software.                           *
*                                                                               *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR    *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,      *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE   *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER        *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, *
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN     *
* THE SOFTWARE.                                                                 *
\*******************************************************************************/
--]]




Levitator = {};

Levitator._NAME			= "Levitator"
Levitator._COPYRIGHT	= "Copyright (C) 2016 Eisbearg"
Levitator._DESCRIPTION	= "Specialization that will levitate vehicles"
Levitator._VERSION		= "1.0.0"




source(Utils.getFilename("inspect.lua", g_currentModDirectory.."scripts/"))




function Levitator.prerequisitesPresent(specializations)
    return true;
end;


function Levitator:load(xmlFile)
	self.toggleLevitator = SpecializationUtil.callSpecializationsFunction("toggleLevitator");
	self.Levitator = {};

	self.isSelectable = true;

	local name = getXMLString(xmlFile, "vehicle.name");
	if name == nil then name = getXMLString(xmlFile, "vehicle.name.en"); end;
	if name == nil then name = getXMLString(xmlFile, "vehicle.name.de"); end;
	if name == nil then name = getXMLString(xmlFile, "vehicle.name.es"); end;
	if name == nil then name = getXMLString(xmlFile, "vehicle.name.fr"); end;
	if name == nil then name = getXMLString(xmlFile, "vehicle.name.pt"); end;
	if name == nil then name = getXMLString(xmlFile, "vehicle.name.ru"); end;
	if name == nil then name = "UnknownVehicle"; end;
	print("--- inserting "..Levitator._NAME.." in vehicle "..name);

	self.Levitator.id = Levitator._NAME;
	self.Levitator.name = name;
	self.Levitator.move = false;
	self.Levitator.lastKey = {};
	self.Levitator.lastKey.symDown = 0;
	self.Levitator.lastKey.modDown = 0;
	self.Levitator.lastKey.symUp = 0;
	self.Levitator.lastKey.modUp = 0;
	self.Levitator.lastKey.scrolllActive = false;
	self.Levitator.lastKey.capsActive = false;
	self.Levitator.active = false;
	self.Levitator.downForce = 0;
	self.Levitator.yForce = 0;
	self.Levitator.dt = 0;
	
	self.lastDigitalSide = 0;
end;


function Levitator:delete()
end;


function Levitator:mouseEvent(posX, posY, isDown, isUp, button)
end;


function Levitator:keyEvent(unicode, sym, modifier, isDown)
	if isDown then
		self.Levitator.lastKey.symDown = sym;
		self.Levitator.lastKey.modDown = modifier;
		
		if sym == 301 and bitAND(modifier, Input.MOD_CAPS) == 0 then
			self.Levitator.lastKey.capsActive = true;
		end;
		if sym == 301 and bitAND(modifier, Input.MOD_CAPS) > 0 then
			self.Levitator.lastKey.capsActive = false;
		end;
		
		if sym == Input.KEY_scrolllock then
			if self.Levitator.lastKey.scrolllActive == false then
				self.Levitator.lastKey.scrolllActive = true;
			else
				self.Levitator.lastKey.scrolllActive = false;
			end;
		end;
		
		if self.Levitator.lastKey.capsActive or self.Levitator.lastKey.scrolllActive then
			if self.Levitator.active == false then Levitator.activateFlightMode(self); end
			self.Levitator.active = true;
		else
			if self.Levitator.active == true then Levitator.deactivateFlightMode(self); end
			self.Levitator.active = false;
		end;
	else
		self.Levitator.lastKey.symUp = sym;
		self.Levitator.lastKey.modUp = modifier;
	end;
end;


function Levitator:activateFlightMode()
	setLinearDamping(self.components[1].node, 0.25);
	setAngularDamping(self.components[1].node, 0.25);
	self.Levitator.downForce = self:getTotalMass(true, false) * 9.8;
	self.Levitator.yForce = self.Levitator.downForce;
	self.cruiseControl.average = (self.cruiseControl.minSpeed + self.cruiseControl.maxSpeed) / 2;
	self.setCruiseControlMaxSpeed(self, self.cruiseControl.average);
	
	local wheelsId = Utils.indexToObject(self.components[1].node, "0>1");
	setVisibility(wheelsId, false);
end;


function Levitator:deactivateFlightMode()
	setLinearDamping(self.components[1].node, 0.0);
	setAngularDamping(self.components[1].node, 0.01);
	self.Levitator.downForce = self:getTotalMass(true, false) * 9.8;
	self.Levitator.yForce = self.Levitator.downForce;
	self.cruiseControl.average = (self.cruiseControl.minSpeed + self.cruiseControl.maxSpeed) / 2;
	self.setCruiseControlMaxSpeed(self, self.cruiseControl.average);
	
	local wheelsId = Utils.indexToObject(self.components[1].node, "0>1");
	setVisibility(wheelsId, true);
end;


function Levitator:update(dt)
	if not self:getIsActiveForInput(true) then return end;
	if not self.Levitator.active then return end;
	
	Levitator.updateMovement(self, dt);
	
	local dX,dY,dZ = localDirectionToWorld(self.components[1].node, 1, 0, 0);
	local sinDir = dX / 1;
	local cosDir = dZ / -1;
	local zSteer = 50 * self.axisForward * -1;
	local xForce = zSteer * cosDir;
	local zForce = zSteer * sinDir;
	
	local ySteer = (self.cruiseControl.speed - self.cruiseControl.average) / self.cruiseControl.average;
	local yForceDiff = self.Levitator.downForce * ySteer * 0.25;
	self.Levitator.yForce = yForceDiff + self.Levitator.downForce;
	
	addForce(self.components[1].node, xForce, self.Levitator.yForce, zForce, 0, 3, 0, true);

	if self.axisSide ~= 0 then
		local AVx, AVy, AVz = getAngularVelocity(self.components[1].node);
		if self.axisSideIsAnalog then
			AVy = self.axisSide * -1;
		else
			AVy = self.lastDigitalSide * -1;
		end;
		setAngularVelocity(self.components[1].node, AVx, AVy, AVz);
	end
	
	self.Levitator.dt = self.Levitator.dt + dt;
	if self.Levitator.dt < 30 then return end;
	self.Levitator.dt = 0;
	
	local AVx, AVy, AVz = getAngularVelocity(self.components[1].node);
	if self.Levitator.lastKey.rx == 0 then
		setAngularVelocity(self.components[1].node, AVx * 0.33, AVy * 0.88 , AVz * 0.33);
	else
		setAngularVelocity(self.components[1].node, AVx * 0.33, AVy,         AVz * 0.33);
	end;
end;


function Levitator:updateTick(dt)
end;


function Levitator:draw()
	if self:getIsActiveForInput(true) then
		g_currentMission:addHelpButtonText(g_i18n:getText(self.Levitator.id), InputBinding[self.Levitator.id]);

		setTextAlignment(RenderText.ALIGN_CENTER);
		setTextBold(true);
		setTextColor(1,1,1,1);

		if self.Levitator.active then
			renderText(0.50, 0.34, 0.016, "move: w s  turn: a d ");
			renderText(0.50, 0.32, 0.016, string.format("pitch: 1 2    current: %3.4f ", self.Levitator.yForce - self.Levitator.downForce));
			renderText(0.50, 0.30, 0.016, string.format("Alt: %4.2f meter", Levitator:getAltitude(self.components[1].node)));
			setTextBold(false);
			renderText(0.50, 0.20, 0.010, string.format("%4.4f sin %1.4f cos %1.4f", Levitator:getMorldDirectionDegree(self.components[1].node)));
		else
			--renderText(0.50, 0.34, 0.016, "fly: Feststelltaste, Shift Lock or Caps Lock");
		end;
		
		--[[
		local locX, locY, locZ = getWorldTranslation(self.components[1].node);
		locY = locY + 0.1;
		drawDebugLine(locX, locY, locZ, 1, 0, 0, locX + 10, locY, locZ, 1, 0, 0);
		drawDebugLine(locX, locY, locZ, 0, 1, 0, locX, locY + 10, locZ, 0, 1, 0);
		drawDebugLine(locX, locY, locZ, 0, 0, 1, locX, locY, locZ + 10, 0, 0, 1);
		--]]

		setTextAlignment(RenderText.ALIGN_LEFT);
		setTextBold(false);
		setTextColor(1,1,1,1);
	end;
end;


function Levitator:toggleLevitator(state, nes)
end;


function Levitator:readStream(streamId, connection)
end;
		
	
function Levitator:writeStream(streamId, connection)
end;


function Levitator:getMorldDirectionDegree(rootNode)
	local dX,dY,dZ = localDirectionToWorld(rootNode, 0, 0, 1);
	local sinus = dX / 1;
	local cosinus = dZ / 1;
	local direction = math.deg(math.atan2(cosinus, sinus));
	return direction, sinus, cosinus;
end;


function Levitator:getAltitude(rootNode)
	local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, getWorldTranslation(rootNode));
	local x, altAboveNN = getWorldTranslation(rootNode);
	local altAboveGround = altAboveNN - terrainHeight;
	if altAboveGround < 0 then altAboveGround = 0.0 end;
	return altAboveGround, altAboveNN, terrainHeight;
end;


function Levitator:updateMovement(dt)
    if not self.isEntered then return; end
    
    local axisForward = InputBinding.getDigitalInputAxis(InputBinding.AXIS_MOVE_FORWARD_VEHICLE);
    if InputBinding.isAxisZero(axisForward) then
        self.axisForward = InputBinding.getAnalogInputAxis(InputBinding.AXIS_MOVE_FORWARD_VEHICLE)
        if not InputBinding.isAxisZero(self.axisForward) then
            self.axisForwardIsAnalog = true;
        end
        self.lastDigitalForward = 0;
    else
        self.axisForward = Utils.clamp(self.lastDigitalForward + dt/self.axisSmoothTime*axisForward, -1, 1);
        self.axisForwardIsAnalog = false;
        self.lastDigitalForward = self.axisForward;
    end;
        
    self.axisSide = InputBinding.getDigitalInputAxis(InputBinding.AXIS_MOVE_SIDE_VEHICLE);
    if InputBinding.isAxisZero(self.axisSide) then
        self.axisSide = InputBinding.getAnalogInputAxis(InputBinding.AXIS_MOVE_SIDE_VEHICLE)
        if not InputBinding.isAxisZero(self.axisSide) then
            self.axisSideIsAnalog = true;
        end
        self.lastDigitalSide = 0;
    else
        self.axisSide = Utils.clamp(self.lastDigitalSide + dt/self.axisSmoothTime*self.axisSide, -1, 1);
        self.axisSideIsAnalog = false;
        self.lastDigitalSide = self.axisSide;
    end;
        
    if not self:getIsActiveForInput(false) then
        if not self.axisSideIsAnalog then
            self.axisSide = 0;
        end
        if not self.axisForwardIsAnalog then
            self.axisForward = 0;
        end
    end
        
    if self.isMotorStarted and self.motorStartTime <= g_currentMission.time then
    end;
    
end;

